/****************************************************************
 Alignment.H
 bmajoros@tigr.org

 Represents an alignment between two sequences (nucleotide, amino
 acid, or other).  The alignment is represented by a list of
 match classifications (MATCH/INSERTION/DELETION) that describe
 the individual points on the path through an alignment matrix 
 (i.e., DIAGONAL/UP/RIGHT).  This is a (relatively) compact 
 representation that is bounded by the sum of the two sequence 
 lengths, but it does not give constant time access to a random 
 base.  It is time-efficient only when processing the entire 
 alignment from left to right.  A constant-time random-access 
 alignment representation could be derived from this.

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Alignment_H
#define INCL_Alignment_H
#include "tigr++/Sequence.H"
#include <iostream>
#include "MatchType.H"
#include "SubstitutionMatrix.H"

class Alignment
{
public:
  static int MAX_WIDTH;
  Alignment(const Sequence &,const Sequence &,Alphabet &,double score=0);
  const Sequence &getFirstSequence() const;
  const Sequence &getSecondSequence() const;
  int getAlignmentLength() const;
  MatchType operator[](int position) const;
  Alignment &operator+=(MatchType);
  double getScore() const;
  void countMismatches(int &mismatches,int &insertions) const;
  int countNearMatches(SubstitutionMatrix<float> &);
  int countNearMatches(SubstitutionMatrix<double> &);
  void getResidualsOnRight(Sequence &,Sequence &);
private:
  const Sequence &s1, &s2;
  TigrVector<MatchType> matchData;
  double score;
  Alphabet &alphabet;
  friend ostream &operator<<(ostream &,const Alignment &);
  void printOn(ostream &) const;
};

ostream &operator<<(ostream &,const Alignment &);

#endif
