#!/usr/bin/perl
use strict;
use FastaWriter;
use SubstitutionMatrix;
use AminoAlphabet;
use LinearRegressor;

my $usage="$0 <length> <iterations> <mutations> <matrix> <gap-open> <gap-extend>\n";
die $usage unless @ARGV==6;
my ($proteinLength,$numIterations,$numMutations,$matrixFile,$gapOpen,$gapExtend)=@ARGV;

my $matrix=new SubstitutionMatrix($matrixFile);
my $writer=new FastaWriter;
my $alphabet=new AminoAlphabet;
my $nAlpha=$alphabet->size();
my ($INSERTION,$DELETION,$SUBSTITUTION)=(0,1,2);

my (@heuristicScores,@smithWatermanScores,@needlemanScores);
for(my $i=0 ; $i<$numIterations ; ++$i)
  {
    my $proteinA=randomProtein($proteinLength);

    # Generate and write out protein A
    my $defline=">a";
    my $filename="a.fasta";
    $writer->writeFasta($defline,$proteinA,$filename);

    my $proteinB=$proteinA;
    for(my $i=0 ; $i<$numMutations ; ++$i)
      {
	# Write out protein B
	$defline=">b";
	$filename="b.fasta";
	$writer->writeFasta($defline,$proteinB,$filename);
	
	# Get full Smith-Waterman score
	my $alignmentDir="/home/bmajoros/GF/alignment";
	my $matrixFilename="$alignmentDir/matrices/blosum62";
	my $command="$alignmentDir/smith-waterman $matrixFilename $gapOpen $gapExtend a.fasta b.fasta PROTEIN";
	my $output=`$command`;
	$output=~/score=(\S+)/ || die "$output\nCOULD NOT PARSE!";
	my $smithWatermanScore=$1;
	push @{$smithWatermanScores[$i]},$smithWatermanScore;

	# Get heuristic score
	$command="$alignmentDir/rapid-comparison $matrixFilename a.fasta b.fasta PROTEIN";
	$output=`$command`;
	$output=~/score=(\S+)/ || die "$output\nCOULD NOT PARSE!";
	my $heuristicScore=$1;
	push @{$heuristicScores[$i]},$heuristicScore;

	# Get Needleman-Wunsch score
	$command="$alignmentDir/needleman $matrixFilename $gapExtend a.fasta b.fasta PROTEIN";
	$output=`$command`;
	$output=~/score=(\S+)/ || die "$output\nCOULD NOT PARSE!";
	my $needlemanScore=$1;
	push @{$needlemanScores[$i]},$needlemanScore;

	# Print out both scores and the mutation distance -> generate the
	# two curves so we can assess the correlation

	# Generate next mutant
	$proteinB=mutate($proteinB);
      }
  }

my $regressor=new LinearRegressor();
my ($slope,$intercept,$r,$r2)=$regressor->regress(\@heuristicScores,\@smithWatermanScores);
print STDERR "y=$slope*x+$intercept\nr=$r r^2=$r2\n";

#goto SKIP;
for(my $i=0 ; $i<$numMutations ; ++$i)
  {
    my $sw=$smithWatermanScores[$i][0];
    my $hr=$heuristicScores[$i][0];
    print "$hr $sw\n";
  }

exit;

SKIP:
for(my $i=0 ; $i<$numMutations ; ++$i)
  {
    my $sw=$smithWatermanScores[$i];
    for(my $j=0 ; $j<$numIterations ; ++$j)
      {
	my $score=$sw->[$j];
	print "$i $score\n";
      }
  }

print "\n";

for(my $i=0 ; $i<$numMutations ; ++$i)
  {
    my $nm=$needlemanScores[$i];
    for(my $j=0 ; $j<$numIterations ; ++$j)
      {
	my $score=$nm->[$j];
	print "$i $score\n";
      }
  }

print "\n";

for(my $i=0 ; $i<$numMutations ; ++$i)
  {
    my $heur=$heuristicScores[$i];
    for(my $j=0 ; $j<$numIterations ; ++$j)
      {
	my $score=400*$heur->[$j];
	print "$i $score\n";
      }
  }

#------------------------------------------------------------------------------
sub randomProtein
  {
    my ($len)=@_;
    my $protein;
    for(my $i=0 ; $i<$len ; ++$i)
      {
	my $symbol=randomAcid();
	while($symbol eq "*") {$symbol=randomAcid()}
	$protein.=$symbol;
      }
    return $protein;
  }
#------------------------------------------------------------------------------
sub randomAcid
  {
    my $r=int(rand($nAlpha));
    return $alphabet->indexToSymbol($r);
  }
#------------------------------------------------------------------------------
sub mutate
  {
    my ($orig)=@_;
    my $len=length $orig;

    # Choose a position
    my $pos=int(rand($len));

    # Decide whether to perform a substitution, deletion, or insertion
    my $action=int(rand(3));
    if($action==$SUBSTITUTION)
      {
	my $symbol;
	while(1)
	  {
	    $symbol=$matrix->chooseLikelySubstitution(substr($orig,$pos,1));
	    last unless $symbol eq "*";
	  }
	substr($orig,$pos,1)=$symbol;
      }
    elsif($action==$INSERTION)
      {
	my $residue;
	while(1)
	  {
	    $residue=$alphabet->indexToSymbol(int(rand($alphabet->size())));
	    last unless $residue eq "*";
	  }
	my $newSeq=substr($orig,0,$pos).$residue.substr($orig,$pos,$len-$pos);
	return $newSeq;
      }
    else # $action==$DELETION
      {
	my $newSeq=substr($orig,0,$pos).substr($orig,$pos+1,$len-$pos-1);
	return $newSeq;
      }
    return $orig;
  }
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------






